@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -213,7 +213,6 @@ namespace IronPython.CodeDom {
                     return;
             }
 
-            Console.WriteLine(reference.BaseType);
             curImports.Add(new CodeNamespaceImport(typeName.Substring(0, typeName.LastIndexOf('.'))));
         }
 
@@ -681,10 +680,10 @@ namespace IronPython.CodeDom {
             AdvanceOutput(e);
 
             Write("return ");
-            GenerateExpression(e.Expression);
-            WriteLine();
-
-            
+            if (e.Expression != null) {
+                GenerateExpression(e.Expression);
+            }
+            WriteLine();            
         }
 
         protected override void GenerateNamespaceEnd(CodeNamespace e) {
@@ -745,6 +744,8 @@ namespace IronPython.CodeDom {
                 if (cni.Namespace == "clr" && cni.UserData["FromImport"] != null) {
                     fHasClr = true;
                 }
+
+                
             }
 
             if (!fHasClr) {
@@ -756,13 +757,14 @@ namespace IronPython.CodeDom {
             AdvanceOutput(e);
 
             string fromImport = e.UserData["FromImport"] as string;
-            if (fromImport != null) {
+            if (e.UserData["Dupped"] != null) {
+                WriteLine(String.Format("from {0} import {1}", e.Namespace, fromImport));
+                WriteLine(String.Format("import {0}", e.Namespace));
+            } else if (fromImport != null) {
                 WriteLine(String.Format("from {0} import {1}", e.Namespace, fromImport));
             } else {
                 WriteLine(String.Format("import {0}", e.Namespace));
-            }
-
-            
+            }            
         }
 
         protected override void GenerateObjectCreateExpression(CodeObjectCreateExpression e) {
@@ -885,10 +887,8 @@ namespace IronPython.CodeDom {
 
             WriteLine("# begin snippet member " + Indent.ToString() + CurrentTypeName);
 
-            string[] lines = e.Text.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.None);
-            foreach (string line in lines) {
-                WriteLine(line);
-            }
+            WriteSnippetWorker(e.Text);
+
             WriteLine("# end snippet member");
 
             Indent = oldIndent;
@@ -896,6 +896,41 @@ namespace IronPython.CodeDom {
             
         }
 
+        /// <summary>
+        /// Checks to see if the given code matches our current indentation level.
+        /// </summary>
+        private bool IsProperlyIndented(string []lines) {            
+            string tabbedIndent = IndentString.Replace("    ", "\t");
+
+            for (int i = 0; i < lines.Length; i++) {
+                for (int j = 0; j < Indent; j++) {
+                    if (lines[i].Length == 0 || lines[i][0] == '#') continue;
+                    
+                    if (lines[i].Length < IndentString.Length*IndentString.Length) return false;
+
+                    int offset = 0;
+                    for (int k = 0; k < Indent; k++) {
+                        if (String.Compare(lines[i], offset, IndentString, 0, IndentString.Length) == 0)
+                            offset += IndentString.Length;
+                        else if (String.Compare(lines[i], offset, tabbedIndent, 0, tabbedIndent.Length) == 0)
+                            offset += tabbedIndent.Length;
+                        else
+                            return false;
+                    }
+                }
+            }
+            return true;
+        }
+
+        private void WriteSnippetWorker(string text) {
+            string[] lines = text.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.None);
+            bool isProperlyIndented = IsProperlyIndented(lines);
+
+            foreach (string line in lines) {
+                WriteLine(line, isProperlyIndented);
+            }
+        }
+
         protected override void GenerateSnippetStatement(CodeSnippetStatement e) {
             AdvanceOutput(e);
             int oldIndent = Indent;
@@ -904,10 +939,8 @@ namespace IronPython.CodeDom {
 
             WriteLine("# Snippet Statement");
 
-            string[] lines = e.Value.Split(new string[] { "\r\n", "\r", "\n" }, StringSplitOptions.None);
-            foreach (string line in lines) {
-                WriteLine(line);
-            }
+            WriteSnippetWorker(e.Value);
+
             WriteLine("# End Snippet Statement");
 
 
@@ -1338,7 +1371,8 @@ namespace IronPython.CodeDom {
             Write("#ExternalSource(\"");
             Write(e.FileName);
             Write("\",");
-            Write(e.LineNumber);
+            // adjust by 1 for the comment we add in for each snippet member or statement
+            Write(e.LineNumber+1);  
             WriteLine(")");
         }
         #endregion
@@ -1700,9 +1734,14 @@ namespace IronPython.CodeDom {
         }
 
         private void Write(string txt) {
+            Write(txt, false);
+        }
+
+        private void Write(string txt, bool preserveSpaces) {
             // enforce consistent indenting - we always use 4 spaces for indentation
             // regardless of what the user provided as this guarantees the code compiles.  The
             // user could provide String.Empty for Indent which breaks Python code.
+
             int prevIndent = Indent;
             Indent = 0;
             string[] lines = txt.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
@@ -1710,10 +1749,12 @@ namespace IronPython.CodeDom {
             for (int i = 0; i < lines.Length; i++) {
                 if (i != 0) {
                     WriteLineTargetBuffer(String.Empty);
-                    for (int j = 0; j < prevIndent; j++) WriteTargetBuffer(IndentString);
-                    col = prevIndent * 4;
+                    if (!preserveSpaces) {
+                        for (int j = 0; j < prevIndent; j++) WriteTargetBuffer(IndentString);
+                        col = prevIndent * 4;
+                    }
                     row++;
-                } else if (col == 0) {
+                } else if (col == 0 && !preserveSpaces) {
                     for (int j = 0; j < prevIndent; j++) WriteTargetBuffer(IndentString);
                     col = prevIndent * 4;
                 }
@@ -1725,6 +1766,10 @@ namespace IronPython.CodeDom {
         }
 
         private void WriteLine(string txt) {
+            WriteLine(txt, false);
+        }
+
+        private void WriteLine(string txt, bool preserveSpaces) {
             // enforce consistent indenting - we always use 4 spaces for indentation
             // regardless of what the user provided as this guarantees the code compiles.  The
             // user could provide String.Empty for Indent which breaks Python code.
@@ -1733,7 +1778,7 @@ namespace IronPython.CodeDom {
             Indent = 0;
             string[] lines = txt.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
             for (int i = 0; i < lines.Length; i++) {
-                if (col == 0) {
+                if (col == 0 && !preserveSpaces) {
                     for (int j = 0; j < prevIndent; j++) WriteTargetBuffer(IndentString);
                 }
 
