@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 
@@ -199,11 +199,19 @@ namespace IronPython.Compiler.Ast {
             cg.Emit(OpCodes.Castclass, typeof(FunctionCode));
             cg.Emit(OpCodes.Dup);
             functionCode.EmitSet(cg);
-            cg.EmitInt(this.Start.Line);
+            if (IsExternal) {
+                cg.EmitInt(this.ExternalStart.Line);
+            } else {
+                cg.EmitInt(this.Start.Line);
+            }
             cg.EmitCall(typeof(FunctionCode), "SetLineNumber");
 
             functionCode.EmitGet(cg);
-            cg.EmitString(this.filename);
+            if (IsExternal) {
+                cg.EmitString(ExternalInfo.OriginalFileName);
+            } else {
+                cg.EmitString(this.filename);
+            }
             cg.EmitCall(typeof(FunctionCode), "SetFilename");
 
             // Only codegen the call into SetFlags if there are flags to set.
@@ -272,17 +280,22 @@ namespace IronPython.Compiler.Ast {
                 PromoteLocalsToEnvironment();
             }
 
-            methodCodeGen.EmitTraceBackTryBlockStart();
-
             // emit the actual body
             if (yieldCount > 0) {
                 EmitGeneratorBody(methodCodeGen, initCodeGen);
             } else {
+                Slot dummySlot = methodCodeGen.GetLocalTmp(typeof(object));
+                methodCodeGen.MarkSequencePointNone();
+                methodCodeGen.EmitTraceBackTryBlockStart(dummySlot);
+                methodCodeGen.FuncOrClassName = name.ToString();
+                methodCodeGen.EmitSetTraceBackUpdateStatus(false);
+
                 EmitFunctionBody(methodCodeGen, initCodeGen);
+
+                methodCodeGen.MarkSequencePointNone();
+                methodCodeGen.FreeLocalTmp(dummySlot);
+                methodCodeGen.EmitTraceBackFaultBlock();
             }
-
-            methodCodeGen.EmitTraceBackFaultBlock(name.GetString(), filename);
-
         }
 
         private bool NeedsWrapperMethod() {
@@ -351,6 +364,8 @@ namespace IronPython.Compiler.Ast {
             ncg.Context = cg.Context;
 
             PromoteLocalsToEnvironment();
+            ncg.FuncOrClassName = name.ToString();
+            ncg.EmitSetTraceBackUpdateStatus(false);
 
             // Namespace without er factory - all locals must exist ahead of time
             ncg.Names = new Namespace(null);
@@ -414,7 +429,11 @@ namespace IronPython.Compiler.Ast {
             for (int i = 0; i < yieldCount; i++) jumpTable[i] = targets[i].TopBranchTarget;
             ncg.yieldLabels = jumpTable;
 
-            ncg.PushTryBlock();
+            // Generator will ofcourse yield, but we are not interested in the isBlockYielded value
+            // hence push a dummySlot to pass the Assertion.
+
+            Slot dummySlot = ncg.GetLocalTmp(typeof(object));
+            ncg.PushTryBlock(dummySlot);
             ncg.BeginExceptionBlock();
 
             ncg.Emit(OpCodes.Ldarg_0);
@@ -424,7 +443,8 @@ namespace IronPython.Compiler.Ast {
             // fall-through on first pass
             // yield statements will insert the needed labels after their returns
             Body.Emit(ncg);
-
+            //free the dummySlot
+            ncg.FreeLocalTmp(dummySlot);
             // fall-through is almost always possible in generators, so this
             // is almost always needed
             ncg.EmitReturnInGenerator(null);
@@ -528,13 +548,11 @@ namespace IronPython.Compiler.Ast {
 
     public struct YieldTarget {
         private Label topBranchTarget;
-        private Label tryBranchTarget;
-        private bool finallyBranch;
+        private Label yieldContinuationTarget;
 
         public YieldTarget(Label topBranchTarget) {
             this.topBranchTarget = topBranchTarget;
-            tryBranchTarget = new Label();
-            finallyBranch = false;
+            yieldContinuationTarget = new Label();
         }
 
         public Label TopBranchTarget {
@@ -542,26 +560,16 @@ namespace IronPython.Compiler.Ast {
             set { topBranchTarget = value; }
         }
 
-        public Label TryBranchTarget {
-            get { return tryBranchTarget; }
-            set { tryBranchTarget = value; }
+        public Label YieldContinuationTarget {
+            get { return yieldContinuationTarget; }
+            set { yieldContinuationTarget = value; }
         }
 
-        public bool FinallyBranch {
-            get { return finallyBranch; }
-            set { finallyBranch = value; }
-        }
-
-        internal YieldTarget FixForTry(CodeGen cg) {
-            tryBranchTarget = cg.DefineLabel();
+        internal YieldTarget FixForTryCatchFinally(CodeGen cg) {
+            yieldContinuationTarget = cg.DefineLabel();
             return this;
         }
 
-        internal YieldTarget FixForFinally(CodeGen cg) {
-            tryBranchTarget = cg.DefineLabel();
-            finallyBranch = true;
-            return this;
-        }
     }
 
     class YieldLabelBuilder : AstWalker {
@@ -569,7 +577,8 @@ namespace IronPython.Compiler.Ast {
             public enum State {
                 Try,
                 Handler,
-                Finally
+                Finally,
+                Else
             };
             public State state;
 
@@ -594,57 +603,46 @@ namespace IronPython.Compiler.Ast {
             }
 
             public override void AddYieldTarget(YieldStatement ys, YieldTarget yt, CodeGen cg) {
-                stmt.AddYieldTarget(yt.FixForTry(cg));
-                ys.Label = yt.TryBranchTarget;
+                stmt.AddYieldTarget(yt.FixForTryCatchFinally(cg));
+                ys.Label = yt.YieldContinuationTarget;
             }
         }
 
         public sealed class TryBlock : ExceptionBlock {
             private TryStatement stmt;
+            private bool isPython24TryFinallyStmt;
 
-            public TryBlock(TryStatement stmt)
-                : this(stmt, State.Try) {
+            public TryBlock(TryStatement stmt, bool isPython24TryFinallyStmt)
+                : this(stmt, State.Try, isPython24TryFinallyStmt) {
             }
-            public TryBlock(TryStatement stmt, State state)
+
+            public TryBlock(TryStatement stmt, State state, bool isPython24TryFinallyStmt)
                 : base(state) {
                 this.stmt = stmt;
+                this.isPython24TryFinallyStmt = isPython24TryFinallyStmt;
             }
 
             public override void AddYieldTarget(YieldStatement ys, YieldTarget yt, CodeGen cg) {
                 switch (state) {
                     case State.Try:
-                        stmt.AddYieldTarget(yt.FixForTry(cg));
-                        ys.Label = yt.TryBranchTarget;
+                        if (isPython24TryFinallyStmt)
+                            cg.Context.AddError("cannot yield from try block with finally", ys);
+                        else
+                            stmt.AddTryYieldTarget(yt.FixForTryCatchFinally(cg));
                         break;
                     case State.Handler:
-                        stmt.YieldInExcept = true;
-                        ys.Label = yt.TopBranchTarget;
-                        break;
-                }
-            }
-        }
-
-        public sealed class TryFinallyBlock : ExceptionBlock {
-            private TryFinallyStatement stmt;
-
-            public TryFinallyBlock(TryFinallyStatement stmt)
-                : this(stmt, State.Try) {
-            }
-            public TryFinallyBlock(TryFinallyStatement stmt, State state)
-                : base(state) {
-                this.stmt = stmt;
-            }
-
-            public override void AddYieldTarget(YieldStatement ys, YieldTarget yt, CodeGen cg) {
-                switch (state) {
-                    case State.Try:
-                        cg.Context.AddError("cannot yield from try block with finally", ys);
+                        stmt.AddCatchYieldTarget(yt.FixForTryCatchFinally(cg));
                         break;
                     case State.Finally:
-                        stmt.AddYieldTarget(yt.FixForFinally(cg));
-                        ys.Label = yt.TryBranchTarget;
+                        stmt.AddFinallyYieldTarget(yt.FixForTryCatchFinally(cg));
                         break;
+                    case State.Else:
+                        stmt.AddElseYieldTarget(yt.FixForTryCatchFinally(cg));
+                        break;
+
                 }
+                ys.Label = yt.YieldContinuationTarget;
+
             }
         }
 
@@ -684,34 +682,38 @@ namespace IronPython.Compiler.Ast {
             return false;
         }
 
-        public override bool Walk(TryFinallyStatement node) {
-            TryFinallyBlock tfb = new TryFinallyBlock(node);
-            tryBlocks.Push(tfb);
-            node.Body.Walk(this);
-            tfb.state = ExceptionBlock.State.Finally;
-            node.FinallyStmt.Walk(this);
-            ExceptionBlock eb = tryBlocks.Pop();
-            Debug.Assert((object)eb == (object)tfb);
-            return false;
-        }
-
 
         public override bool Walk(TryStatement node) {
-            TryBlock tb = new TryBlock(node);
+            TryBlock tb = null;
+
+            if (!Options.Python25 && node.Handlers == null)
+                tb = new TryBlock(node, true);
+            else
+                tb = new TryBlock(node, false);
+
             tryBlocks.Push(tb);
             node.Body.Walk(this);
 
-            tb.state = TryBlock.State.Handler;
-            foreach (TryStatementHandler handler in node.Handlers) {
-                handler.Walk(this);
+            if (node.Handlers != null) {
+                tb.state = TryBlock.State.Handler;
+                foreach (TryStatementHandler handler in node.Handlers) {
+                    handler.Walk(this);
+                }
+            }
+
+            if (node.ElseStatement != null) {
+                tb.state = TryBlock.State.Else;
+                node.ElseStatement.Walk(this);
+            }
+
+            if (node.FinallyStatement != null) {
+                tb.state = TryBlock.State.Finally;
+                node.FinallyStatement.Walk(this);
             }
 
             ExceptionBlock eb = tryBlocks.Pop();
             Debug.Assert((object)tb == (object)eb);
 
-            if (node.ElseStatement != null) {
-                node.ElseStatement.Walk(this);
-            }
             return false;
         }
 
