@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -293,7 +293,7 @@ namespace IronPython.Compiler.Generation {
             AssemblyGen ag = OutputGenerator.Snippets;
 
             string name = GetName();
-            tg = ag.DefinePublicType(TypePrefix + name, baseType);
+            tg = ag.DefinePublicType(TypePrefix + name.Replace('+', '-'), baseType);
 
             ImplementInterfaces();
 
@@ -398,6 +398,33 @@ namespace IronPython.Compiler.Generation {
                 string methodName = mi.Name;
                 if (methodName.StartsWith("#base#")) {
                     string newName = GetBaseName(mi, specialNames);
+
+                    // don't store __ names for extensible types.  super will dispatch to the
+                    // non-virtual method declard on the Ops type, and adding these will just slow
+                    // down the normal calls in this case.  For non-extensible types we need to add this
+                    // so we dispatch w/o stack overflowing on super calls.
+                    if (!mi.DeclaringType.FullName.StartsWith(TypePrefix + "IronPython.Runtime.Operations.Extensible")) {
+                        ParameterInfo[] pis = mi.GetParameters();
+                        Type[] types = new Type[pis.Length];
+                        int i = 0;
+                        foreach (ParameterInfo pi in pis) {
+                            types[i++] = pi.ParameterType;
+                        }
+                        MethodInfo overriding = this.baseType.GetMethod(methodName.Substring(6),
+                            BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance,
+                            null,
+                            types,
+                            null);
+                        Debug.Assert(overriding != null);
+
+                        
+                        string outName;
+                        NameType nt = NameConverter.TryGetName(Ops.GetDynamicTypeFromType(baseType) as ReflectedType, overriding, out outName);
+                        if (nt != NameType.None && outName != newName) {
+                            rt.StoreReflectedBaseMethod(outName, mi, nt);
+                        }
+                    }
+
                     rt.StoreReflectedBaseMethod(newName, mi, NameType.Method);
                 }
             }
