@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -103,11 +103,7 @@ namespace IronPython.Hosting {
 
         public static string VersionString {
             get {
-                Version pythonVersion = Version;
-                string version = String.Format("IronPython {0}.{1}.{2} on .NET {3}",
-                                  pythonVersion.Major, pythonVersion.Minor, pythonVersion.Build,
-                                  Environment.Version);
-                return version;
+                return String.Format("{0} ({1}) on .NET {2}", GetInformationalVersion(), GetFileVersion(), Environment.Version);
             }
         }
 
@@ -165,7 +161,7 @@ namespace IronPython.Hosting {
         }
 
         public void InitializeModules(string prefix, string executable, string version) {
-            Sys.version = version;
+            Sys.SetVersion(version);
             Sys.prefix = prefix;
             Sys.executable = executable;
             Sys.exec_prefix = prefix;
@@ -498,8 +494,8 @@ namespace IronPython.Hosting {
             }
         }
 
-        public bool ParseInteractiveInput(string text, bool allowIncompleteStatement) {
-            Parser p = Parser.FromString(Sys, new CompilerContext("<stdin>"), text);
+        public bool ParseInteractiveInput(string text, bool allowIncompleteStatement) {            
+            Parser p = Parser.FromString(Sys, DefaultCompilerContext("<stdin>"), text);
             return VerifyInteractiveInput(p, allowIncompleteStatement);
         }
 
@@ -785,7 +781,7 @@ namespace IronPython.Hosting {
             Expression e = p.ParseTestListAsExpression();
             ReturnStatement ret = new ReturnStatement(e);
             int lineCnt = expression.Split('\n').Length;
-            ret.SetLoc(new Location(lineCnt, 0), new Location(lineCnt, 10));
+            ret.SetLoc(null, new Location(lineCnt, 0), new Location(lineCnt, 10));
             CodeGen cg = CreateDelegateWorker<TDelegate>(ret, parameters);
 
             return delegate(EngineModule engineModule) {
@@ -824,11 +820,16 @@ namespace IronPython.Hosting {
                 }
             }
 
+            Slot dummySlot = null;
             // finally emit the function into the method, if we
             // have ref/out params then we wrap it in a try/finally
             // that updates them before the function ends.
             if (fixers != null) {
-                cg.PushTryBlock();
+                // Try block may yield, but we are not interested in the isBlockYielded value
+                // hence push a dummySlot to pass the Assertion.
+                dummySlot = cg.GetLocalTmp(typeof(object));
+
+                cg.PushTryBlock(dummySlot);
                 cg.BeginExceptionBlock();
             }
 
@@ -837,7 +838,7 @@ namespace IronPython.Hosting {
             if (fixers != null) {
                 cg.PopTargets();
                 Slot returnVar = cg.GetLocalTmp(typeof(int));
-                cg.PushFinallyBlock(returnVar);
+                cg.PushFinallyBlock(returnVar, dummySlot);
                 cg.BeginFinallyBlock();
 
                 foreach (ReturnFixer rf in fixers) {
@@ -846,6 +847,7 @@ namespace IronPython.Hosting {
 
                 cg.EndExceptionBlock();
                 cg.PopTargets();
+                cg.FreeLocalTmp(dummySlot);
             }
             cg.Finish();
             return cg;
@@ -1042,7 +1044,27 @@ namespace IronPython.Hosting {
         }
 
         #endregion
-    }
 
+        private static T GetAssemblyAttribute<T>() where T : Attribute {
+            Assembly asm = typeof(PythonEngine).Assembly;
+            object[] attributes = asm.GetCustomAttributes(typeof(T), false);
+            if (attributes != null && attributes.Length > 0) {
+                return (T)attributes[0];
+            } else {
+                Debug.Fail(String.Format("Cannot find attribute {0}", typeof(T).Name));
+                return null;
+            }
+        }
+
+        private static string GetInformationalVersion() {
+            AssemblyDescriptionAttribute attribute = GetAssemblyAttribute<AssemblyDescriptionAttribute>();
+            return attribute != null ? attribute.Description : "";
+        }
+
+        private static string GetFileVersion() {
+            AssemblyFileVersionAttribute attribute = GetAssemblyAttribute<AssemblyFileVersionAttribute>();
+            return attribute != null ? attribute.Version : "";
+        }
+    }
 }
 
