@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Text;
@@ -32,6 +32,7 @@ using IronPython.Runtime.Operations;
 using IronPython.Hosting;
 
 using IronMath;
+using System.IO;
 
 [assembly: PythonModule("__builtin__", typeof(IronPython.Modules.Builtin))]
 namespace IronPython.Modules {
@@ -83,6 +84,23 @@ namespace IronPython.Modules {
             if (ret == null) {
                 throw Ops.ImportError("No module named {0}", name);
             }
+
+            PythonModule mod = ret as PythonModule;
+            if (mod != null && from != null) {
+                string strAttrName;
+                object attrValue;
+                foreach (object attrName in from) {
+                    if (Converter.TryConvertToString(attrName, out strAttrName) && strAttrName != null) {
+                        try {
+                            attrValue = Ops.ImportOneFrom(context, mod, strAttrName);
+                        } catch (PythonImportErrorException) {
+                            continue;
+                        }
+                        mod.SetImportedAttr(context, SymbolTable.StringToId(strAttrName), attrValue);
+                    }
+                }
+            }
+
             return ret;
         }
 
@@ -126,6 +144,12 @@ namespace IronPython.Modules {
             return Ops.FALSE;
         }
 
+        [PythonName("apply")]
+        [Documentation("apply(object[, args[, kwargs]]) -> value\n\nDeprecated.\nInstead, use:\n    function(*args, **keywords).")]
+        public static object Apply(ICallerContext context, object func) {
+            return Ops.CallWithContext(context, func);
+        }
+
         [PythonName("apply")]
         [Documentation("apply(object[, args[, kwargs]]) -> value\n\nDeprecated.\nInstead, use:\n    function(*args, **keywords).")]
         public static object Apply(ICallerContext context, object func, object args) {
@@ -243,7 +267,9 @@ namespace IronPython.Modules {
             } else {
                 throw Ops.ValueError("compile() arg 3 must be 'exec' or 'eval' or 'single'");
             }
-            return new FunctionCode(compiledCode, cflags);
+            FunctionCode res = new FunctionCode(compiledCode, cflags);
+            res.SetFilename(filename);
+            return res;
         }
 
         [PythonName("compile")]
@@ -514,9 +540,13 @@ namespace IronPython.Modules {
             BuiltinFunction bf;
             PythonFunction pf;
             BuiltinMethodDescriptor methodDesc;
+            BoundBuiltinFunction bbf;
             string strVal;
             PythonModule pm;
             OldClass oc;
+            ReflectedField rf;
+            ReflectedProperty rp;
+            ReflectedEvent re;
 
             if (doced.Contains(o)) return;  // document things only once
             doced.Add(o);
@@ -587,6 +617,14 @@ namespace IronPython.Modules {
                     if (dt.TryGetSlot(context, SymbolTable.StringToId(name), out value))
                         Help(context, doced, doc, indent + 1, value);
                 }
+            } else if ((bbf = o as BoundBuiltinFunction) != null) {
+                if (indent == 0) doc.AppendFormat("Help on built-in function {0}\n\n", bbf.Target.Name);
+
+                AppendIndent(doc, indent);
+                doc.Append(bbf.Target.Name);
+                doc.Append("(...)\n");
+
+                AppendMultiLine(doc, bbf.Target.Documentation, indent + 1);
             } else if ((methodDesc = o as BuiltinMethodDescriptor) != null) {
                 if (indent == 0) doc.AppendFormat("Help on method-descriptor {0}\n\n", methodDesc.Name);
                 AppendIndent(doc, indent);
@@ -601,11 +639,52 @@ namespace IronPython.Modules {
                 doc.Append("(...)\n");
 
                 AppendMultiLine(doc, bf.Documentation, indent + 1);
+            } else if ((rf = o as ReflectedField) != null) {
+                if (indent == 0) doc.AppendFormat("Help on field {0}\n\n", rf.Name);
+                AppendIndent(doc, indent);
+                doc.Append("Field ");
+                doc.AppendLine(rf.Name);
+
+                if (rf.Documentation != null) AppendMultiLine(doc, rf.Documentation, indent + 1);
+            } else if ((rp = o as ReflectedProperty) != null) {
+                if (indent == 0) doc.AppendFormat("Help on property {0}\n\n", rp.Name);
+                AppendIndent(doc, indent);
+                doc.Append("Property ");
+                doc.AppendLine(rp.Name);
+
+                if (rp.Documentation != null) AppendMultiLine(doc, rp.Documentation, indent + 1);
+            } else if ((re = o as ReflectedEvent) != null) {
+                if (indent == 0) doc.AppendFormat("Help on event {0}\n\n", re.Name);
+                AppendIndent(doc, indent);
+                doc.Append("Event ");
+                doc.AppendLine(re.Name);
+
+                if (re.Documentation != null) AppendMultiLine(doc, re.Documentation, indent + 1);
             } else if ((pf = o as PythonFunction) != null) {
                 if (indent == 0) doc.AppendFormat("Help on function {0} in module {1}\n\n", pf.Name, pf.Module.ModuleName);
 
                 AppendIndent(doc, indent);
-                doc.AppendFormat("{0}({1})\n", pf.Name, String.Join(", ", pf.ArgNames));
+                FunctionCode fc = pf.FunctionCode as FunctionCode;
+
+                int specialCnt = 0;
+                if ((fc.Flags & (int)FunctionCode.FuncCodeFlags.VarArgs) != 0) specialCnt++;
+                if ((fc.Flags & (int)FunctionCode.FuncCodeFlags.KwArgs) != 0) specialCnt++;
+
+                doc.Append(pf.Name);
+                doc.Append('(');
+                doc.Append(String.Join(", ", pf.ArgNames, 0, pf.ArgNames.Length - specialCnt));
+                if ((fc.Flags & (int)FunctionCode.FuncCodeFlags.VarArgs) != 0) {
+                    if (pf.ArgNames.Length - specialCnt != 0) doc.Append(", ");
+                    doc.Append('*');
+                    doc.Append(pf.ArgNames[pf.ArgNames.Length - specialCnt]);
+                }
+                if ((fc.Flags & (int)FunctionCode.FuncCodeFlags.KwArgs) != 0) {
+                    if (pf.ArgNames.Length - 1 != 0) doc.Append(", ");
+                    doc.Append("**");
+                    doc.Append(pf.ArgNames[pf.ArgNames.Length-1]);
+                }
+                doc.Append(")\n");
+
                 string pfDoc = Converter.ConvertToString(pf.Documentation);
                 if (!String.IsNullOrEmpty(pfDoc)) {
                     AppendMultiLine(doc, pfDoc, indent);
@@ -1246,10 +1325,26 @@ namespace IronPython.Modules {
 
             if (module.Filename == null) return Importer.ReloadBuiltin(module);
 
-            CompilerContext cc = new CompilerContext(module.Filename);
-            Parser parser = Parser.FromFile(module.SystemState, cc);
-            Statement s = parser.ParseFileInput();
-            PythonModule pmod = OutputGenerator.GenerateModule(module.SystemState, cc, s, module.ModuleName, "__" + System.Threading.Interlocked.Increment(ref reloadCounter));
+            PythonModule pmod = null;
+
+            bool reloadBinary = false;
+            bool binaryLoaded = string.Compare(Path.GetExtension(module.Filename), ".exe", true) == 0;
+
+            if (binaryLoaded) {
+                string source = Path.ChangeExtension(module.Filename, ".py");
+                reloadBinary = Importer.ShouldLoadPreCompiledModule(module.Filename, source);
+            }
+
+            if (reloadBinary) {
+                pmod = Importer.LoadPreCompiled(module.SystemState, module.ModuleName, module.Filename);
+            }
+
+            if (pmod == null) {
+                CompilerContext cc = new CompilerContext(module.Filename);
+                Parser parser = Parser.FromFile(module.SystemState, cc);
+                Statement s = parser.ParseFileInput();
+                pmod = OutputGenerator.GenerateModule(module.SystemState, cc, s, module.ModuleName, "__" + System.Threading.Interlocked.Increment(ref reloadCounter));
+            }
 
             foreach (KeyValuePair<object, object> attr in module.__dict__) {
                 if (pmod.__dict__.ContainsObjectKey(attr.Key)) continue;
