@@ -1,23 +1,25 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
-using System.IO;
+using System.Collections;
+using System.Collections.Specialized;
 using System.Diagnostics;
-using System.Security.Cryptography;
+using System.IO;
 using System.Runtime.InteropServices;
+using System.Security.Cryptography;
 
 using IronMath;
 using IronPython.Runtime;
@@ -136,7 +138,7 @@ namespace IronPython.Modules {
 
         [PythonName("mkdir")]
         public static void MakeDirectory(string path) {
-            if (Directory.Exists(path)) throw Ops.IOError("directory already exists");
+            if (Directory.Exists(path)) throw Ops.OSError("directory already exists");
 
             try {
                 Directory.CreateDirectory(path);
@@ -147,7 +149,7 @@ namespace IronPython.Modules {
 
         [PythonName("mkdir")]
         public static void MakeDirectory(string path, int mode) {
-            if (Directory.Exists(path)) throw Ops.IOError("directory already exists");
+            if (Directory.Exists(path)) throw Ops.OSError("directory already exists");
             // we ignore mode
 
             try {
@@ -327,30 +329,52 @@ namespace IronPython.Modules {
 
         [PythonName("spawnl")]
         public static object SpawnProcess(int mode, string path, params object[] args) {
-            System.Diagnostics.Process process;
+            return SpawnProcessImpl(MakeProcess(), mode, path, args);
+        }
+
+        [PythonName("spawnle")]
+        public static object SpawnProcessWithParamsArgsAndEnvironment(int mode, string path, params object[] args) {
+            if (args.Length < 1) {
+                throw Ops.TypeError("spawnle() takes at least three arguments ({0} given)", 2 + args.Length);
+            }
+
+            object env = args[args.Length - 1];
+            object[] slicedArgs = new object[args.Length - 1];
+
+            Array.Copy(args, 1, slicedArgs, 0, args.Length - 1);
+
+            Process process = MakeProcess();
+            SetEnvironment(process.StartInfo.EnvironmentVariables, env);
+
+            return SpawnProcessImpl(process, mode, path, slicedArgs);
+        }
+
+        [PythonName("spawnv")]
+        public static object SpawnProcess(int mode, string path, object args) {
+            return SpawnProcessImpl(MakeProcess(), mode, path, args);
+        }
+
+        [PythonName("spawnve")]
+        public static object SpawnProcess(int mode, string path, object args, object env) {
+            Process process = MakeProcess();
+            SetEnvironment(process.StartInfo.EnvironmentVariables, env);
+
+            return SpawnProcessImpl(process, mode, path, args);
+        }
+
+        private static Process MakeProcess() {
             try {
-                process = new System.Diagnostics.Process();
+                return new Process();
+            } catch (Exception e) {
+                throw ToPythonException(e);
+            }
+        }
+
+        private static object SpawnProcessImpl(Process process, int mode, string path, object args) {
+            try {
+                process.StartInfo.Arguments = ArgumentsToString(args);
                 process.StartInfo.FileName = path;
                 process.StartInfo.UseShellExecute = false;
-                if (args != null && args.Length > 0) {
-                    System.Text.StringBuilder sb = new System.Text.StringBuilder();
-                    bool space = false;
-                    foreach (object arg in args) {
-                        string strarg = Ops.ToString(arg);
-                        if (space) {
-                            sb.Append(' ');
-                        }
-                        if (strarg.IndexOf(' ') != -1) {
-                            sb.Append('"');
-                            sb.Append(strarg);
-                            sb.Append('"');
-                        } else {
-                            sb.Append(strarg);
-                        }
-                        space = true;
-                    }
-                    process.StartInfo.Arguments = sb.ToString();
-                }
             } catch (Exception e) {
                 throw ToPythonException(e);
             }
@@ -358,7 +382,7 @@ namespace IronPython.Modules {
             if (!process.Start()) {
                 throw Ops.OSError("Cannot start process: {0}", path);
             }
-            if (mode == 0) {
+            if (mode == (int)P_WAIT) {
                 process.WaitForExit();
                 int exitCode = process.ExitCode;
                 process.Close();
@@ -368,6 +392,67 @@ namespace IronPython.Modules {
             }
         }
 
+        /// <summary>
+        /// Copy elements from a Python mapping of dict environment variables to a StringDictionary.
+        /// </summary>
+        private static void SetEnvironment(StringDictionary currentEnvironment, object newEnvironment) {
+            Dict env = newEnvironment as Dict;
+            if (env == null) {
+                throw Ops.TypeError("env argument must be a dict");
+            }
+
+            currentEnvironment.Clear();
+
+            string strKey, strValue;
+            foreach (object key in env.Keys) {
+                if (!Converter.TryConvertToString(key, out strKey)) {
+                    throw Ops.TypeError("env dict contains a non-string key");
+                }
+                if (!Converter.TryConvertToString(env[key], out strValue)) {
+                    throw Ops.TypeError("env dict contains a non-string value");
+                }
+                currentEnvironment[strKey] = strValue;
+            }
+        }
+
+        /// <summary>
+        /// Convert a sequence of args to a string suitable for using to spawn a process.
+        /// </summary>
+        private static string ArgumentsToString(object args) {
+            IEnumerator argsEnumerator;
+            System.Text.StringBuilder sb = null;
+            if (!Converter.TryConvertToIEnumerator(args, out argsEnumerator)) {
+                throw Ops.TypeError("args parameter must be sequence, not {0}", Ops.GetDynamicType(args));
+            }
+
+            bool space = false;
+            try {
+                // skip the first element, which is the name of the command being run
+                argsEnumerator.MoveNext();
+                while (argsEnumerator.MoveNext()) {
+                    if (sb == null) sb = new System.Text.StringBuilder(); // lazy creation
+                    string strarg = Ops.ToString(argsEnumerator.Current);
+                    if (space) {
+                        sb.Append(' ');
+                    }
+                    if (strarg.IndexOf(' ') != -1) {
+                        sb.Append('"');
+                        sb.Append(strarg);
+                        sb.Append('"');
+                    } else {
+                        sb.Append(strarg);
+                    }
+                    space = true;
+                }
+            } finally {
+                IDisposable disposable = argsEnumerator as IDisposable;
+                if (disposable != null) disposable.Dispose();
+            }
+
+            if (sb == null) return "";
+            return sb.ToString();
+        }
+
         [PythonName("startfile")]
         public static void StartFile(string filename) {
             System.Diagnostics.Process process = new System.Diagnostics.Process();
@@ -678,11 +763,11 @@ namespace IronPython.Modules {
                     fi.LastAccessTime = DateTime.Now;
                     fi.LastWriteTime = DateTime.Now;
                 } else if (times.Count == 2) {
-                    DateTime atime = new DateTime(1969, 12, 31, 16, 0, 0).Add(TimeSpan.FromSeconds(Converter.ConvertToDouble(times[0])));
-                    DateTime mtime = new DateTime(1969, 12, 31, 16, 0, 0).Add(TimeSpan.FromSeconds(Converter.ConvertToDouble(times[1])));
+                    DateTime atime = DateTime.MinValue.Add(TimeSpan.FromSeconds(Converter.ConvertToDouble(times[0])));
+                    DateTime mtime = DateTime.MinValue.Add(TimeSpan.FromSeconds(Converter.ConvertToDouble(times[1])));
 
                     fi.LastAccessTime = atime;
-                    fi.LastAccessTime = mtime;
+                    fi.LastWriteTime = mtime;
                 } else {
                     throw Ops.TypeError("times value must be a 2-value tuple (atime, mtime)");
                 }
@@ -731,6 +816,13 @@ namespace IronPython.Modules {
         public static object O_BINARY = 0x8000;
         public static object O_TEXT = 0x4000;
 
+        public static object P_WAIT = 0;
+        public static object P_NOWAIT = 1;
+        public static object P_NOWAITO = 3;
+        // Not implemented:
+        // public static object P_OVERLAY = 2;
+        // public static object P_DETACH = 4;
+
         #endregion
 
         #region Private implementation details
@@ -800,27 +892,36 @@ namespace IronPython.Modules {
         }
 
         private static ProcessStartInfo GetProcessInfo(string command) {
-            ProcessStartInfo psi;
-            string baseCommand = command;
-            string args = null;
-            bool fInQuote = false;
-            int argStart = command.Length;
+            command = command.Trim();
 
-            for (int i = 0; i < command.Length; i++) {
-                if (command[i] == '\"') fInQuote = !fInQuote;
-                else if (!fInQuote && command[i] == ' ') {
-                    argStart = i + 1;
-                    break;
+            string baseCommand = command;
+            string args = string.Empty;
+            int pos;
+
+            if (command[0] == '\"') {
+                for (pos = 1; pos < command.Length; pos++) {
+                    if (command[pos] == '\"') {
+                        baseCommand = command.Substring(1, pos - 1).Trim();
+                        if (pos + 1 < command.Length) {
+                            args = command.Substring(pos + 1);
+                        }
+                        break;
+                    }
+                }
+                if (pos == command.Length)
+                    throw Ops.ValueError("mismatch quote in command");
+            } else {
+                pos = command.IndexOf(' ');
+                if (pos != -1) {
+                    baseCommand = command.Substring(0, pos);
+                    // pos won't be the last one
+                    args = command.Substring(pos + 1);
                 }
             }
 
-            if (argStart < command.Length) {
-                args = command.Substring(argStart);
-                baseCommand = command.Substring(0, argStart - 1);
-            }
-
             baseCommand = GetCommandFullPath(baseCommand);
-            psi = new ProcessStartInfo(baseCommand, args);
+
+            ProcessStartInfo psi = new ProcessStartInfo(baseCommand, args);
             psi.UseShellExecute = false;
 
             return psi;
