@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Text;
@@ -23,6 +23,7 @@ using System.Diagnostics;
 using IronPython.Runtime;
 using IronPython.Runtime.Operations;
 using IronPython.Runtime.Exceptions;
+using System.Runtime.InteropServices;
 
 [assembly: PythonModule("re", typeof(IronPython.Modules.PythonRegex))]
 namespace IronPython.Modules {
@@ -118,6 +119,10 @@ namespace IronPython.Modules {
             ValidateString(@string, "string");
 
             MatchCollection mc = pat.FindAllWorker(@string, 0, @string.Length);
+            return FixFindAllMatch(pat, mc);
+        }
+
+        private static object FixFindAllMatch(RE_Pattern pat, MatchCollection mc) {
             object[] matches = new object[mc.Count];
             int numgrps = pat.re.GetGroupNumbers().Length;
             for (int i = 0; i < mc.Count; i++) {
@@ -130,7 +135,7 @@ namespace IronPython.Modules {
                     //  in the example above, ('', 'x') did not match (\d+) as indicated by '' but did 
                     //  match (\w+) as indicated by 'x' and so on...
                     int k = 0;
-                    ArrayList tpl = new ArrayList();
+                    List<object> tpl = new List<object>();
                     foreach (Group g in mc[i].Groups) {
                         //  here also the CLR gives us a "bonus" match as the first item which is the 
                         //  group that was actually matched in the tuple e.g. we get 'x', '', 'x' for 
@@ -152,7 +157,6 @@ namespace IronPython.Modules {
                     matches[i] = mc[i].Value;
                 }
             }
-
             return new List(matches);
         }
 
@@ -193,9 +197,9 @@ namespace IronPython.Modules {
             return Split(ValidatePattern(pattern), ValidateString(@string, "string"), 0);
         }
         [PythonName("split")]
-        public static object Split(object pattern, object @string, int maxSplit) {
+        public static object Split(object pattern, object @string, int maxsplit) {
             return new RE_Pattern(ValidatePattern(pattern)).Split(ValidateString(@string, "string"),
-                maxSplit);
+                maxsplit);
         }
 
         [PythonName("sub")]
@@ -250,23 +254,34 @@ namespace IronPython.Modules {
             [PythonName("match")]
             public RE_Match Match(object text) {
                 string input = ValidateString(text, "text");
-                return RE_Match.makeMatch(re.Match(input), this, input, 0);
+                return RE_Match.makeMatch(re.Match(input), this, input, 0, input.Length);
+            }
+
+            private static int FixPosition(string text, int position) {
+                if (position < 0) return 0;
+                if (position > text.Length) return text.Length;
+
+                return position;
             }
 
             [PythonName("match")]
-            public RE_Match Match(object text, int pos) {
+            public RE_Match Match(object text, int pos) {                
                 string input = ValidateString(text, "text");
-                return RE_Match.makeMatch(re.Match(input, pos), this, input, pos);
+                pos = FixPosition(input, pos);
+                return RE_Match.makeMatch(re.Match(input, pos), this, input, pos, input.Length);
             }
 
             [PythonName("match")]
-            public RE_Match Match(object text, int pos, int endpos) {
+            public RE_Match Match(object text, [DefaultParameterValue(0)] int pos, int endpos) {                
                 string input = ValidateString(text, "text");
+                pos = FixPosition(input, pos);
+                endpos = FixPosition(input, endpos);
                 return RE_Match.makeMatch(
                     re.Match(input.Substring(0, endpos), pos),
                     this,
                     input,
-                    pos);
+                    pos,
+                    endpos);
             }
 
             [PythonName("search")]
@@ -301,12 +316,7 @@ namespace IronPython.Modules {
             public object FindAll(object @string, int pos, object endpos) {
                 MatchCollection mc = FindAllWorker(ValidateString(@string, "text"), pos, endpos);
 
-                object[] matches = new object[mc.Count];
-                for (int i = 0; i < mc.Count; i++) {
-                    matches[i] = mc[i].Value;
-                }
-
-                return new List(matches);
+                return new List(FixFindAllMatch(this, mc));
             }
 
             internal MatchCollection FindAllWorker(string str, int pos, object endpos) {
@@ -342,10 +352,10 @@ namespace IronPython.Modules {
             }
 
             [PythonName("split")]
-            public object Split(object @string, int maxSplit) {
+            public object Split(object @string, int maxsplit) {
                 List result = new List();
                 // fast path for negative maxSplit ( == "make no splits")
-                if (maxSplit < 0)
+                if (maxsplit < 0)
                     result.AddNoLock(@string);
                 else {
                     // iterate over all matches
@@ -354,20 +364,22 @@ namespace IronPython.Modules {
                     int lastPos = 0; // is either start of the string, or first position *after* the last match
                     int nSplits = 0; // how many splits have occurred?
                     foreach (Match m in matches) {
-                        // add substring from lastPos to beginning of current match
-                        result.AddNoLock(theStr.Substring(lastPos, m.Index - lastPos));
-                        // if there are subgroups of the match, add their match or None
-                        if (m.Groups.Count > 1)
-                            for (int i = 1; i < m.Groups.Count; i++)
-                                if (m.Groups[i].Success)
-                                    result.AddNoLock(m.Groups[i].Value);
-                                else
-                                    result.AddNoLock(null);
-                        // update lastPos, nSplits
-                        lastPos = m.Index + m.Length;
-                        nSplits++;
-                        if (nSplits == maxSplit)
-                            break;
+                        if (m.Length > 0) {
+                            // add substring from lastPos to beginning of current match
+                            result.AddNoLock(theStr.Substring(lastPos, m.Index - lastPos));
+                            // if there are subgroups of the match, add their match or None
+                            if (m.Groups.Count > 1)
+                                for (int i = 1; i < m.Groups.Count; i++)
+                                    if (m.Groups[i].Success)
+                                        result.AddNoLock(m.Groups[i].Value);
+                                    else
+                                        result.AddNoLock(null);
+                            // update lastPos, nSplits
+                            lastPos = m.Index + m.Length;
+                            nSplits++;
+                            if (nSplits == maxsplit)
+                                break;
+                        }
                     }
                     // add tail following last match
                     result.AddNoLock(theStr.Substring(lastPos));
@@ -470,8 +482,11 @@ namespace IronPython.Modules {
                         Dict d = new Dict();
                         string[] names = re.GetGroupNames();
                         int[] nums = re.GetGroupNumbers();
-                        for (int i = 1; i < names.Length; i++)
+                        for (int i = 1; i < names.Length; i++) {
+                            if (Char.IsDigit(names[i][0])) continue;    // skip numeric names
+
                             d[names[i]] = nums[i];
+                        }
                         groups = d;
                     }
                     return groups;
@@ -508,6 +523,8 @@ namespace IronPython.Modules {
             RE_Pattern pattern;
             private Match m;
             private string text;
+            private int lastindex = -1;
+            private int endPos;
 
             #region Internal makers
             internal static RE_Match make(Match m, RE_Pattern pattern, string input) {
@@ -515,8 +532,8 @@ namespace IronPython.Modules {
                 return null;
             }
 
-            internal static RE_Match makeMatch(Match m, RE_Pattern pattern, string input, int offset) {
-                if (m.Success && m.Index == offset) return new RE_Match(m, pattern, input);
+            internal static RE_Match makeMatch(Match m, RE_Pattern pattern, string input, int offset, int endPos) {
+                if (m.Success && m.Index == offset) return new RE_Match(m, pattern, input, endPos);
                 return null;
             }
             #endregion
@@ -526,11 +543,12 @@ namespace IronPython.Modules {
                 this.m = m;
                 this.pattern = pattern;
                 this.text = text;
-                for (int i = 0; i < m.Groups.Count; i++) {
-                    if (m.Groups[i].Captures.Count > 0) {
-                        lastindex = i;
-                    }
-                }
+            }
+            public RE_Match(Match m, RE_Pattern pattern, string text, int endpos) {
+                this.m = m;
+                this.pattern = pattern;
+                this.text = text;
+                this.endPos = endpos;
             }
             #endregion
 
@@ -612,8 +630,6 @@ namespace IronPython.Modules {
                 return Ops.MakeTuple(ret);
             }
 
-            public object lastindex;
-
             [PythonName("expand")]
             public object Expand(object template) {
                 string strTmp = ValidateString(template, "template");
@@ -655,13 +671,20 @@ namespace IronPython.Modules {
                 return GroupDict(null);
             }
 
+            private static bool IsGroupNumber(string name) {
+                foreach (char c in name) {
+                    if (!Char.IsNumber(c)) return false;
+                }
+                return true;
+            }
+
             [PythonName("groupdict")]
             public object GroupDict(object value) {
                 string[] groupNames = this.pattern.re.GetGroupNames();
                 Debug.Assert(groupNames.Length == this.m.Groups.Count);
                 Dict d = new Dict();
                 for (int i = 0; i < groupNames.Length; i++) {
-                    if (groupNames[i] == "0") continue; // python doesn't report this overarching group
+                    if (IsGroupNumber(groupNames[i])) continue; // python doesn't report group numbers
 
                     if (m.Groups[i].Captures.Count != 0) {
                         d[groupNames[i]] = m.Groups[i].Value;
@@ -674,13 +697,12 @@ namespace IronPython.Modules {
 
             [PythonName("span")]
             public object Span() {
-                return Tuple.MakeTuple(m.Groups[0].Index, m.Groups[0].Index + m.Groups[0].Length);
+                return Tuple.MakeTuple(this.Start(), this.End());
             }
 
             [PythonName("span")]
             public object Span(object group) {
-                int groupInt = GetGroupIndex(group);
-                return Tuple.MakeTuple(m.Groups[groupInt].Index, m.Groups[groupInt].Index + m.Groups[groupInt].Length);
+                return Tuple.MakeTuple(this.Start(group), this.End(group));
             }
 
             public int Position {
@@ -693,7 +715,7 @@ namespace IronPython.Modules {
             public int EndPosition {
                 [PythonName("endpos")]
                 get {
-                    return m.Index + m.Length;
+                    return endPos;
                 }
             }
 
@@ -707,16 +729,73 @@ namespace IronPython.Modules {
             public object Regs {
                 [PythonName("regs")]
                 get {
-                    // what is this?
-                    return Tuple.MakeTuple(Tuple.MakeTuple(0, 1), Tuple.MakeTuple(0, 1));
+                    object[] res = new object[m.Groups.Count];
+                    for(int i = 0; i<res.Length; i++) {
+                        res[i] = Tuple.MakeTuple(this.Start(i), this.End(i));
+                    }
+                                        
+                    return Tuple.MakeTuple(res);
                 }
             }
+
             public object Pattern {
                 [PythonName("re")]
                 get {
                     return pattern;
                 }
             }
+
+            public object LastIndex {
+                [PythonName("lastindex")]
+                get {
+                    //   -1 : initial value of lastindex
+                    //    0 : no match found
+                    //other : the true lastindex
+
+                    // Match.Groups contains "lower" level matched groups, which has to be removed
+                    if (lastindex == -1) {
+                        int i = 1;
+                        while (i < m.Groups.Count) {
+                            if (m.Groups[i].Success) {
+                                lastindex = i;
+                                int start = m.Groups[i].Index;
+                                int end = start + m.Groups[i].Length;
+                                i++;
+
+                                // skip any group which fall into the range [start, end], 
+                                // no matter match succeed or fail
+                                while (i < m.Groups.Count && (m.Groups[i].Index < end)) {
+                                    i++;
+                                }
+                            } else {
+                                i++;
+                            }
+                        }
+
+                        if (lastindex == -1) {
+                            lastindex = 0;
+                        }
+                    }
+
+                    if (lastindex == 0) {
+                        return null;
+                    } else {
+                        return lastindex;
+                    }
+                }
+            }
+
+            public object LastGroup {
+                [PythonName("lastgroup")]
+                get {
+                    if (LastIndex == null) return null;
+
+                    // when group was not explicitly named, RegEx assigns the number as name
+                    // This is different from C-Python, which returns None in such cases
+
+                    return this.pattern.re.GroupNameFromNumber((int)LastIndex);
+                }
+            }
             #endregion
 
             #region Private helper functions
@@ -887,6 +966,10 @@ namespace IronPython.Modules {
                     case 'Z':
                         // known escape sequences, leave escaped.
                         break;
+                    case '\\':
+                        // escaping a \\
+                        cur += 2;
+                        break;
                     default:
                         System.Globalization.UnicodeCategory charClass = Char.GetUnicodeCategory(curChar);
                         switch (charClass) {
@@ -934,7 +1017,6 @@ namespace IronPython.Modules {
                                 case 'r': sb.Append('\r'); break;
                                 case 't': sb.Append('\t'); break;
                                 case '\\': sb.Append('\\'); break;
-                                case '?': sb.Append('?'); break;
                                 case '\'': sb.Append('\''); break;
                                 case '"': sb.Append('"'); break;
                                 case 'b': sb.Append('\b'); break;
