@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections.Generic;
@@ -144,8 +144,8 @@ namespace IronPython.Modules {
         [PythonName("unpack")]
         public static Tuple Unpack(string fmt, string @string) {
             string data = @string;
-            int count = 1;
-            int curIndex = 0;
+            int count = 1;              // formats can be pre-pended w/ a repeat count, this is that count
+            int curIndex = 0;           // current index into the output string.
             List<object> res = new List<object>();
             bool fLittleEndian = BitConverter.IsLittleEndian;
             bool fStandardized = false;
@@ -154,7 +154,8 @@ namespace IronPython.Modules {
             for (int i = 0; i < fmt.Length; i++) {
                 switch (fmt[i]) {
                     case 'x': // pad byte
-                        curIndex++;
+                        curIndex+=count;
+                        count = 1;
                         break;
                     case 'c': // char
                         for (int j = 0; j < count; j++) res.Add(CreateCharValue(ref curIndex, data).ToString());
