@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using IronMath;
@@ -658,7 +658,19 @@ namespace IronPython.Runtime {
             object deleg = Ops.GetDelegate(value, to);
             if (deleg != null) return deleg;
 
-            throw Ops.TypeErrorForBadInstance("expected compatible function, found {0}", value);
+            if (Ops.IsCallable(value)) {
+                int min, max;
+                int expectedArgs = to.GetMethod("Invoke").GetParameters().Length;
+                if (!Ops.IsCallableCompatible(value, expectedArgs, out min, out max)) {
+                    if (min == max) {
+                        throw Ops.TypeError("expected compatible function, but got parameter count mismatch (expected {0} args, target takes {1})", expectedArgs, min);
+                    } else {
+                        throw Ops.TypeError("expected compatible function, but got parameter count mismatch (expected {0} args, target takes at least {1} and at most {2})", expectedArgs, min, max);
+                    }
+                }
+            } 
+            throw Ops.TypeErrorForBadInstance("expected compatible function, but found non-callable object of type {0}", value);
+            
         }
 
 
