@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -192,7 +192,7 @@ namespace IronPython.Runtime {
                 lock (this) if (TryGetValue(realKey, out ret)) return ret;
                 if (Options.Python25 && Ops.TryInvokeSpecialMethod(this, SymbolTable.Missing, out ret, key))
                     return ret;
-                throw Ops.KeyError("'{0}'", key);
+                throw Ops.KeyError(key);
             }
             set {
                 lock (this) data[DictOps.NullToObj(key)] = value;
@@ -209,12 +209,12 @@ namespace IronPython.Runtime {
         #region IPythonContainer Members
 
         [PythonName("__len__")]
-        public int GetLength() {
+        public virtual int GetLength() {
             return DictOps.Length(this);
         }
 
         [PythonName("__contains__")]
-        public bool ContainsValue(object value) {
+        public virtual bool ContainsValue(object value) {
             return DictOps.Contains(this, value);
         }
 
@@ -463,7 +463,7 @@ namespace IronPython.Runtime {
         }
 
         int ICollection.Count {
-            get { return data.Count; }
+            get { return GetLength(); }
         }
 
         bool ICollection.IsSynchronized {
@@ -652,7 +652,7 @@ namespace IronPython.Runtime {
 
         public static void DelIndex(IDictionary<object, object> self, object key) {
             if (!self.Remove(key)) {
-                throw Ops.KeyError("'{0}'", key);
+                throw Ops.KeyError(key);
             }
         }
 
@@ -679,7 +679,7 @@ namespace IronPython.Runtime {
                 self.Remove(key);
                 return ret;
             } else {
-                throw Ops.KeyError("'{0}'", key);
+                throw Ops.KeyError(key);
             }
         }
 
