@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections.Generic;
@@ -47,6 +47,7 @@ namespace IronPython.Runtime.Exceptions {
         static PythonFunction exceptionInitMethod;
         static PythonFunction exceptionGetItemMethod;
         static PythonFunction exceptionStrMethod;
+        static PythonFunction exceptionGetStateMethod;
         static PythonFunction syntaxErrorStrMethod;
         static PythonFunction unicodeErrorInit;
         static PythonFunction systemExitInitMethod;
@@ -119,6 +120,7 @@ namespace IronPython.Runtime.Exceptions {
                     new ExceptionMapping("OverflowWarning",typeof(PythonOverflowWarningException)),
                     new ExceptionMapping("RuntimeWarning",typeof(PythonRuntimeWarningException)),
                     new ExceptionMapping("FutureWarning",typeof(PythonFutureWarningException)),
+                    new ExceptionMapping("ImportWarning",typeof(PythonImportWarningException)),
                 }),
             })            
         };
@@ -127,6 +129,7 @@ namespace IronPython.Runtime.Exceptions {
             exceptionInitMethod = new FunctionX(null, "__init__", new CallTargetN(ExceptionConverter.ExceptionInit), new string[] { "args" }, Ops.EMPTY, FunctionAttributes.ArgumentList);
             exceptionGetItemMethod = new FunctionX(null, "__getitem__", new CallTargetN(ExceptionConverter.ExceptionGetItem), new string[] { "args" }, Ops.EMPTY, FunctionAttributes.ArgumentList);
             exceptionStrMethod = new FunctionX(null, "__str__", new CallTargetN(ExceptionConverter.ExceptionToString), new string[] { "args" }, Ops.EMPTY, FunctionAttributes.ArgumentList);
+            exceptionGetStateMethod= new FunctionX(null, "__getstate__", new CallTargetN(ExceptionConverter.ExceptionGetState), new string[] { "args" }, Ops.EMPTY, FunctionAttributes.ArgumentList);
             syntaxErrorStrMethod = new FunctionX(null, "__str__",
                 new CallTargetN(ExceptionConverter.SyntaxErrorToString), new string[] { "args" }, Ops.EMPTY, FunctionAttributes.ArgumentList);
             unicodeErrorInit = new FunctionX(null,
@@ -192,7 +195,7 @@ namespace IronPython.Runtime.Exceptions {
             ExceptionInit(new Tuple(false, new object[] { self, encoding, @object, start, end, reason }));
 
             Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("encoding"), encoding);
-            Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("@object"), @object);
+            Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("object"), @object);
             Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("start"), start);
             Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("end"), end);
             Ops.SetAttr(DefaultContext.Default, self, SymbolTable.StringToId("reason"), reason);
@@ -266,6 +269,33 @@ namespace IronPython.Runtime.Exceptions {
             return String.Empty;
         }
 
+        /// <summary>
+        /// Helper function for exception instances. Returns pickled object state.
+        // ie. Exception.__getstate__
+        /// </summary>
+        public static Dict ExceptionGetState(params object[] args) {
+            Debug.Assert(args.Length == 1);
+            Tuple t = args[0] as Tuple;
+
+            if (t == null || t.GetLength() == 0) throw Ops.TypeErrorForUnboundMethodCall("__getstate__", typeof(Exception), null);
+
+            object self = t[0];
+
+            if (!IsExceptionObject(self))
+                throw Ops.TypeErrorForUnboundMethodCall("__getstate__", typeof(Exception), self);
+
+            if (t.GetLength() != 1) throw Ops.TypeErrorForArgumentCountMismatch("__getstate__", 1, t.GetLength());
+
+            OldInstance selfObj = (OldInstance)self;
+            Dict dictCopy = new Dict(selfObj.GetAttrDict(DefaultContext.Default).Count - 1);
+            foreach (KeyValuePair<object, object> item in selfObj.GetAttrDict(DefaultContext.Default)) {
+                if (item.Key is string && (string)item.Key == SymbolTable.ClrExceptionKey.ToString()) continue;
+                dictCopy[item.Key] = item.Value;
+            }
+
+            return dictCopy;
+        }
+
         /// <summary>
         /// Helper ToString function for SyntaxError instances.  Returns
         /// line information in addition to just the mesage.
@@ -597,6 +627,7 @@ namespace IronPython.Runtime.Exceptions {
             oc.SetAttr(DefaultContext.Default, SymbolTable.Init, exceptionInitMethod);
             oc.SetAttr(DefaultContext.Default, SymbolTable.GetItem, exceptionGetItemMethod);
             oc.SetAttr(DefaultContext.Default, SymbolTable.String, exceptionStrMethod);
+            oc.SetAttr(DefaultContext.Default, SymbolTable.GetState, exceptionGetStateMethod);
 
             return oc;
         }
