@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -355,8 +355,10 @@ namespace IronPython.Runtime {
         }
 
         internal static Type[] LoadTypesFromAssembly(Assembly asm) {
-            if (asm is System.Reflection.Emit.AssemblyBuilder) {
-                // assembly builders don't support GetExportedTypes, always call GetTypes on them.
+            try {
+                return Compiler.Options.PrivateBinding ? asm.GetTypes() : asm.GetExportedTypes();
+            } catch (NotSupportedException) {
+                // AssemblyBuilders don't support GetExportedTypes
                 if (!Compiler.Options.PrivateBinding) {
                     return Array.FindAll(asm.GetTypes(), delegate(Type t) {
                         return t.IsPublic;
@@ -364,7 +366,6 @@ namespace IronPython.Runtime {
                 }
                 return asm.GetTypes();
             }
-            return Compiler.Options.PrivateBinding ? asm.GetTypes() : asm.GetExportedTypes();
         }
 
         #endregion
