@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections.Generic;
@@ -328,6 +328,8 @@ namespace IronPython.Runtime {
         public static readonly SymbolId Extend = new SymbolId(ExtendId);
         ///<summary>Symbol for 'update'</summary> 
         public static readonly SymbolId Update = new SymbolId(UpdateId);
+        ///<summary>Symbol for 'func_name'</summary> 
+        public static readonly SymbolId FunctionName = new SymbolId(FunctionNameId);
 
         // *** END GENERATED CODE ***
 
@@ -495,7 +497,8 @@ namespace IronPython.Runtime {
         public const int AppendId                 = 146; // "append"
         public const int ExtendId                 = 147; // "extend"
         public const int UpdateId                 = 148; // "update"
-        public const int LastWellKnownId          = 149; // "LastWellKnown"
+        public const int FunctionNameId           = 149; // "func_name"
+        public const int LastWellKnownId          = 150; // "LastWellKnown"
 
         // *** END GENERATED CODE ***
 
@@ -663,6 +666,7 @@ namespace IronPython.Runtime {
             PublishWellKnownSymbol("append", Append);  // 146
             PublishWellKnownSymbol("extend", Extend);  // 147
             PublishWellKnownSymbol("update", Update);  // 148
+            PublishWellKnownSymbol("func_name", FunctionName);  // 149
 
             // *** END GENERATED CODE ***
 
