@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Collections;
@@ -36,6 +36,7 @@ namespace IronPython.Runtime {
 
     [PythonType(typeof(PythonModule))]
     public class SystemState : ICustomAttributes, IDisposable {
+        [PythonHiddenField]
         public Encoding DefaultEncoding;
 
         private IAttributesDictionary __dict__;
@@ -129,20 +130,43 @@ namespace IronPython.Runtime {
             DefaultEncoding = Encoding.ASCII;
             byteorder = BitConverter.IsLittleEndian ? "little" : "big";
             copyright = "Copyright (c) Microsoft Corporation. All rights reserved.";
-            hexversion = 0x02040000;
             maxint = Int32.MaxValue;
             maxunicode = (int)ushort.MaxValue;
             platform = "cli";
             version_info = Tuple.MakeTuple(2, 4, 0, "final", 0);    // report as being compatible w/ 2.4.0 final
+            winver = "2.4";
             // !!! These fields do need to be reset on "reload(sys)". However, the initial value is specified by the 
             // engine elsewhere. For now, we initialize them just once to some default value
             if (version == null) {
-                version = IronPython.Hosting.PythonEngine.VersionString;
+                SetVersion(PythonEngine.VersionString);
                 warnoptions = List.Make();
                 executable = "";
             }
         }
 
+        internal void SetVersion(string ironPythonVersion) {
+            Version pythonVersion;
+            string pythonVersionLevel;
+
+            if (Options.Python25) {
+                pythonVersion = new Version(2, 5, 0);
+                pythonVersionLevel = "alpha";
+            } else {
+                pythonVersion = new Version(2, 4, 0);
+                pythonVersionLevel = "release";
+            }
+
+            Debug.Assert(pythonVersion.Major <= 0xff, "version components must fit in two-hex-digit field");
+            Debug.Assert(pythonVersion.Minor <= 0xff, "version components must fit in two-hex-digit field");
+            Debug.Assert(pythonVersion.Build <= 0xff, "version components must fit in two-hex-digit field");
+
+            // e.g. hexversion = 0x02040000;
+            hexversion = (pythonVersion.Major << 24) + (pythonVersion.Minor << 16) + (pythonVersion.Build << 8);
+            version_info = Tuple.MakeTuple(pythonVersion.Major, pythonVersion.Minor, pythonVersion.Build, pythonVersionLevel, 0);
+
+            version = String.Format("{0}.{1}.{2} ({3})", pythonVersion.Major, pythonVersion.Minor, pythonVersion.Build, ironPythonVersion);
+        }
+
         public override string ToString() {
             return "<module 'sys' (built-in)>";
         }
@@ -237,6 +261,11 @@ namespace IronPython.Runtime {
 
         [PythonName("exc_clear")]
         public void ClearException() {
+            ThreadAbortException tae = SystemState.RawException as ThreadAbortException;
+            if (tae != null && tae.ExceptionState is PythonKeyboardInterruptException) {
+                Thread.ResetAbort();
+            }
+
             SystemState.RawException = null;
             SystemState.RawTraceBack = null;
         }
@@ -396,6 +425,9 @@ namespace IronPython.Runtime {
         [PythonName("version_info")]
         public object version_info;
 
+        [PythonName("winver")]
+        public object winver;
+
         #region ICustomAttributes Members
 
         public bool TryGetAttr(ICallerContext context, SymbolId name, out object value) {
