@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.Text;
@@ -88,6 +88,19 @@ namespace IronPython.Runtime.Types {
             return string.Format("<field# {0} on {1}>", info.Name, info.DeclaringType.Name);
         }
 
+        public string Documentation {
+            [PythonName("__doc__")]
+            get {
+                return ReflectionUtil.CreateAutoDoc(info);
+            }
+        }
+
+        public string Name {
+            get {
+                return info.Name;
+            }
+        }
+
         #region IContextAwareMember Members
 
         public bool IsVisible(ICallerContext context) {
@@ -204,7 +217,7 @@ namespace IronPython.Runtime.Types {
             }
         }
 
-        protected PropertyInfo Info {
+        internal PropertyInfo Info {
             get {
                 return info;
             }
@@ -523,6 +536,18 @@ namespace IronPython.Runtime.Types {
             return string.Format("<event# {0} on {1}>", dispatcher.Info.Name, dispatcher.Info.DeclaringType.Name);
         }
 
+        public string Name {
+            get {
+                return dispatcher.Info.Name;
+            }
+        }
+
+        public string Documentation {
+            [PythonName("__doc__")]
+            get {
+                return ReflectionUtil.CreateAutoDoc(dispatcher.Info);
+            }
+        }
         #region IContextAwareMember Members
 
         bool IContextAwareMember.IsVisible(ICallerContext context) {
