@@ -1,17 +1,17 @@
-/* **********************************************************************************
+/* ****************************************************************************
  *
- * Copyright (c) Microsoft Corporation. All rights reserved.
+ * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Shared Source License
- * for IronPython. A copy of the license can be found in the License.html file
- * at the root of this distribution. If you can not locate the Shared Source License
- * for IronPython, please send an email to ironpy@microsoft.com.
- * By using this source code in any fashion, you are agreeing to be bound by
- * the terms of the Shared Source License for IronPython.
+ * This source code is subject to terms and conditions of the Microsoft Public
+ * License. A  copy of the license can be found in the License.html file at the
+ * root of this distribution. If  you cannot locate the  Microsoft Public
+ * License, please send an email to  dlr@microsoft.com. By using this source
+ * code in any fashion, you are agreeing to be bound by the terms of the 
+ * Microsoft Public License.
  *
  * You must not remove this notice, or any other, from this software.
  *
- * **********************************************************************************/
+ * ***************************************************************************/
 
 using System;
 using System.IO;
@@ -180,7 +180,7 @@ namespace IronPythonConsole {
                 } else {
                     return RunFile(engine, fileName);
                 }
-            } catch (System.Threading.ThreadAbortException tae) {
+            } catch (System.Threading.ThreadAbortException tae) {                
                 if (tae.ExceptionState is PythonKeyboardInterruptException) {
                     Thread.ResetAbort();
                 }
@@ -208,6 +208,7 @@ namespace IronPythonConsole {
                     case "-X:Frames": Options.Frames = true; break;
                     case "-X:GenerateAsSnippets": Options.GenerateModulesAsSnippets = true; break;
                     case "-X:ILDebug": Options.ILDebug = true; break;
+                    case "-X:NotImportCompiled": Options.ImportCompiledModule = false; break;
                     case "-c":
                         args.RemoveAt(0);
                         if (args.Count == 0) {
@@ -314,7 +315,7 @@ namespace IronPythonConsole {
             Console.WriteLine(PythonEngine.Copyright);
             Console.WriteLine("Usage: ipy [options] [file.py|- [arguments]]");
             Console.WriteLine("Options:");
-            Console.WriteLine("  -O:                    Enable optimizations");
+            Console.WriteLine("  -O                     Enable optimizations");
 #if DEBUG
             Console.WriteLine("  -D                     EngineDebug mode");
 #endif
@@ -329,6 +330,7 @@ namespace IronPythonConsole {
             Console.WriteLine("  -X:Frames              Generate custom frames");
             Console.WriteLine("  -X:GenerateAsSnippets  Generate code to run in snippet mode");
             Console.WriteLine("  -X:ILDebug             Output generated IL code to a text file for debugging");
+            Console.WriteLine("  -X:NotImportCompiled   Never import pre-compiled modules");
             Console.WriteLine("  -X:MaxRecursion        Set the maximum recursion level");
             Console.WriteLine("  -X:MTA                 Run in multithreaded apartment");
             Console.WriteLine("  -X:NoOptimize          Disable optimized methods");
