@@ -1,17 +1,17 @@
 #####################################################################################
 #
-#  Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation. 
 #
-#  This source code is subject to terms and conditions of the Shared Source License
-#  for IronPython. A copy of the license can be found in the License.html file
-#  at the root of this distribution. If you can not locate the Shared Source License
-#  for IronPython, please send an email to ironpy@microsoft.com.
-#  By using this source code in any fashion, you are agreeing to be bound by
-#  the terms of the Shared Source License for IronPython.
+# This source code is subject to terms and conditions of the Microsoft Public
+# License. A  copy of the license can be found in the License.html file at the
+# root of this distribution. If  you cannot locate the  Microsoft Public
+# License, please send an email to  dlr@microsoft.com. By using this source
+# code in any fashion, you are agreeing to be bound by the terms of the 
+# Microsoft Public License.
 #
-#  You must not remove this notice, or any other, from this software.
+# You must not remove this notice, or any other, from this software.
 #
-######################################################################################
+#####################################################################################
 
 '''
 IronPython Test Driver
@@ -64,6 +64,58 @@ def getNextResultLog():
 
     raise AssertionError, 'cannot create log file'
 
+## To control the output stream
+#
+#   - NullStream eats all, 
+#   - MyFileStream flushes to file immediately (More consideration)
+#
+
+class NullStream:
+    softspace = False
+    def __init__(self): pass
+    def __repr__(self): return ''
+    def close(self):    pass
+    def flush(self):    pass
+    def fileno(self):   return 1
+    def read(self):     return ""
+    def readline(self): return "\n"
+    def write(self, s): pass
+
+
+class MyFileStream(NullStream):
+    def __init__(self, sw): self.sw = sw
+    def close(self):    self.sw.close()
+    def write(self, s):
+        self.sw.write(s)
+        self.sw.flush()        
+
+logname, logfile = getNextResultLog()
+logstream = MyFileStream(logfile)
+
+assertOccurred = False
+
+if is_cli: 
+    from System.Diagnostics import Debug, TraceListener
+    
+    class MyTraceListener(TraceListener):
+        def __init__(self, stream):
+            self.stream = stream
+            self.banner = '\n!!!' + 'X' * 70 + '!!!\n'
+        def Write(self, message):
+            self.stream.write(self.banner) 
+            self.stream.write(message)
+            self.stream.write(self.banner) 
+            global assertOccurred
+            assertOccurred = True
+        def WriteLine(self, message):
+            self.Write(message + r'\n')
+        def Flush(self):
+            self.stream.flush()
+            
+    Debug.Listeners.Clear()
+    myListener = MyTraceListener(logstream)
+    Debug.Listeners.Add(myListener)
+
 # result related classes
 class TestResultSet:
     startTime = time.ctime()
@@ -161,6 +213,11 @@ class TestResult:
         self.succeed = True
         self.exception = None
         self._stopTest()
+    
+    def setAssertOccur(self):
+        self.succeed = False
+        self.exception = None
+        self._stopTest()    
 ##
 ## How each file gets run
 ##
@@ -212,31 +269,7 @@ def LibraryRunStep(file, timeLevel='long'):
 def CompatRunStep(file, timelevel='med'):
     package = __import__(file)
     del sys.modules[file]
-    
-## To control the output stream
-#
-#   - NullStream eats all, 
-#   - MyFileStream flushes to file immediately (More consideration)
-#
 
-class NullStream:
-    softspace = False
-    def __init__(self): pass
-    def __repr__(self): return ''
-    def close(self):    pass
-    def flush(self):    pass
-    def fileno(self):   return 1
-    def read(self):     return ""
-    def readline(self): return "\n"
-    def write(self, s): pass
-
-
-class MyFileStream(NullStream):
-    def __init__(self, sw): self.sw = sw
-    def close(self):    self.sw.close()
-    def write(self, s):
-        self.sw.write(s)
-        self.sw.flush()        
 
 ## Runners
 ## not used, but leave it for debugging purpose
@@ -277,16 +310,27 @@ class RedirectTestRunner(TestRunner):
         self._redirect_output()
         if self.detailLevel <> 'min':
             self.saved_stdout.write(test.ljust(formatter.TestNameLen))
-        logstream.write('>>>> ' + test)
+        logstream.write('>>>> ' + test +'\n')
         try:
             self.testResult.startTest()
             self.runstep(test, self.timeLevel)
-            self.testResult.setSuccess()
+            
             logstream.write('\n') 
-            if self.detailLevel == 'min':
-                self.saved_stdout.write(".")
+            global assertOccurred
+            if assertOccurred: 
+                self.testResult.setAssertOccur()
+                assertOccurred = False
+
+                if self.detailLevel == 'min':
+                    self.saved_stdout.write("A(%s)" % test)
+                else:
+                    self.saved_stdout.write("!ASSERT!\n")
             else:
-                self.saved_stdout.write(" PASS \n")
+                self.testResult.setSuccess()
+                if self.detailLevel == 'min':
+                    self.saved_stdout.write(".")
+                else:
+                    self.saved_stdout.write(" PASS \n")
         except Exception, e: 
             logstream.write('\t\t*FAIL*\n')
             self.testResult.setFailure((str(e.args), my_format_exc()))
@@ -302,8 +346,7 @@ class RedirectTestRunner(TestRunner):
 # test configuration
 class TestConfig: 
     def __init__(self):
-        self.notRunList = [ 
-        ]        
+        self.notRunList = [ 'test_cominterop', 'test_superconsole']        
         self.name       = 'IronPythonTests'
         self.shortcut   = 'ip iron ironpython'
         self.directory  = testpath.public_testdir
@@ -404,6 +447,20 @@ class TestConfig:
                 self.testList.extend([x for x in reqs if x.startswith("test_")])
         return self.testList
 
+class MathTestConfig(TestConfig):
+    def __init__(self):
+        self.notRunList = ['nztest.testFactor', ]
+        self.name       = "Math"
+        self.shortcut   = 'math nzmath'
+        self.directory  = [testpath.math_testdir, testpath.lib_testdir,]
+        self.runner     = RedirectTestRunner
+        self.runstep    = RegressRunStep        
+        self.categories = categories.MathTests
+        self.Mode       = 1
+        
+    def getAllTests(self):
+        return self.getTestsShownInCategories()
+
 class MiscTestConfig(TestConfig):
     def __init__(self):
         self.notRunList = []
@@ -416,6 +473,21 @@ class MiscTestConfig(TestConfig):
         
     def getAllTests(self):
         return self.getTestsShownInCategories()
+        
+        
+class ConsoleInputTestConfig(TestConfig):
+    def __init__(self):
+        self.notRunList = [ ]        
+        self.name       = 'ConsoleInputTests'
+        self.shortcut   = 'consinp'
+        self.directory  = testpath.public_testdir
+        self.runstep    = ImportRunStep
+        self.runner     = RedirectTestRunner
+        self.categories = categories.ConsoleInputTests
+        
+        
+    def getAllTests(self):
+        return self.getTestsShownInCategories()
 
 class LibraryTestConfig(TestConfig):
     def __init__(self):
@@ -429,7 +501,7 @@ class LibraryTestConfig(TestConfig):
 
     def getAllTests(self):
         return self.getTestsShownInCategories()
-    
+
 class RegressionTestConfig(TestConfig):
     def __init__(self):
         self.notRunList = []
@@ -449,6 +521,8 @@ class RegressionTestConfig(TestConfig):
 class CompatTestConfig(TestConfig):
     def __init__(self):
         self.notRunList = ['sbs_typeop.py']
+        if is_cli64:
+            self.notRunList.append('sbs_exceptions.py')
         self.name       = "Compatability"
         self.shortcut   = 'compat sbs'
         self.directory  = testpath.compat_testdir
@@ -464,7 +538,8 @@ class CompatTestConfig(TestConfig):
        
 def getAllConfigs(exclude=[]):
     _module = sys.modules[__name__]
-    return [ getattr(_module, x)() for x in dir(_module) if x.endswith("TestConfig") and not exclude.count(x[:-10])]
+    ret_val = [ getattr(_module, x)() for x in dir(_module) if x.endswith("TestConfig") and not exclude.count(x[:-10])]
+    return ret_val
 
 ## not used. leave it for future                    
 def getAllConfigs2():       
@@ -488,14 +563,6 @@ def getAllConfigs2():
 
     return l
 
-logname, logfile = getNextResultLog()
-logstream = MyFileStream(logfile)
-
-#if is_cli: 
-    #from System.Diagnostics import Debug
-    #Debug.Listeners.Clear()
-    #Debug.Listeners.Add(
-
 def main(args):
     unknown = [x for x in args if x.startswith('-') and not x.startswith('-T:') and not x.startswith('-O:')]
     if unknown: usage(1, 'unknown options: %s' % unknown)
