@@ -1,17 +1,17 @@
 #####################################################################################
 #
-#  Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation. 
 #
-#  This source code is subject to terms and conditions of the Shared Source License
-#  for IronPython. A copy of the license can be found in the License.html file
-#  at the root of this distribution. If you can not locate the Shared Source License
-#  for IronPython, please send an email to ironpy@microsoft.com.
-#  By using this source code in any fashion, you are agreeing to be bound by
-#  the terms of the Shared Source License for IronPython.
+# This source code is subject to terms and conditions of the Microsoft Public
+# License. A  copy of the license can be found in the License.html file at the
+# root of this distribution. If  you cannot locate the  Microsoft Public
+# License, please send an email to  dlr@microsoft.com. By using this source
+# code in any fashion, you are agreeing to be bound by the terms of the 
+# Microsoft Public License.
 #
-#  You must not remove this notice, or any other, from this software.
+# You must not remove this notice, or any other, from this software.
 #
-######################################################################################
+#####################################################################################
 
 from lib.assert_util import *
 from lib.type_util import *
@@ -74,6 +74,7 @@ def test_set_dict():
     Assert(o2.__class__ is C)
     Assert(o2.__class__ is not o1.__class__)
 
+
 ############################################################
 def test_attrs():
     class C:pass
@@ -318,6 +319,34 @@ def test_inheritance_attrs_dir():
     Assert(bz.__module__ == [2, 3 , 4])
 
 
+############################################################
+def test_oldstyle_setattr():
+    global called
+    class C:
+        def __setattr__(self, name, value):
+            global called
+            called = (self, name, value)
+            
+    a = C()
+    a.abc = 'def'
+    AreEqual(called, (a, 'abc', 'def'))
+    
+    del C.__setattr__
+    
+    a.qrt = 'abc'
+    
+    AreEqual(called, (a, 'abc', 'def'))
+    
+    def setattr(self, name, value): 
+        global called
+        called = (self, name, value)        
+    
+    C.__setattr__ = setattr
+    
+    a.qrt = 'abc'
+    
+    AreEqual(called, (a, 'qrt', 'abc'))
+    
 ############################################################
 def test_oldstyle_getattr():
     """verify we don't access __getattr__ while creating an old
@@ -1429,6 +1458,14 @@ if is_cli:
     def test_type_mro():
         AssertError(NotImplementedError, type.mro, int)
 
+def test_derived_tuple_eq():
+    # verify overriding __eq__ on tuple still allows us to call the super version
+    class bazbar(tuple):    
+        def __eq__(self,other):
+            other = bazbar(other)
+            return super(bazbar,self).__eq__(other)
+    AreEqual(bazbar('abc'), 'abc')
+    
 def test_new_old_slots():
     class N(object): pass
     class O: pass
@@ -1510,6 +1547,22 @@ def test_slots_multiple():
         __slots__=()
     C().x = "hello"
     
+def test_override_container_methods():
+    for x in (dict, list, tuple):
+        class C(x):
+            def __len__(self): return 42
+            def __contains__(self, other): 
+                return other == "abc"
+            
+        AreEqual(len(C()), 42)
+        AreEqual('abc' in C(), True)
+        
+def test_dictproxy_access():
+    def f():
+        int.__dict__[0] = 0
+        
+    AssertError(TypeError, f)
+
 # tests w/ special requirements that can't be run in methods..
 #Testing the class attributes backed by globals
     
@@ -1585,4 +1638,17 @@ def test_fastnew_int():
     AreEqual(int(C5()), -123456789012345678910)
     for x in [C6, C7]:      AssertError(TypeError, int, x())       
 
+
+def test_usertype_call():
+    class F(object):
+        def __call__(self, *args, **kwargs):
+                return args, kwargs
+        
+    f = F()
+    AreEqual(f(*(1,), **{'a':23}), ((1,), {'a': 23}))
+    AreEqual(f(a = 7, *(1,)), ((1,), {'a': 7}))
+
+def test_type_call():
+    AreEqual(list.__call__(), [])
+    AreEqual(list.__call__(sequence='abc'), ['a', 'b', 'c'])
 run_test(__name__)
