@@ -1,17 +1,17 @@
 #####################################################################################
 #
-#  Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation. 
 #
-#  This source code is subject to terms and conditions of the Shared Source License
-#  for IronPython. A copy of the license can be found in the License.html file
-#  at the root of this distribution. If you can not locate the Shared Source License
-#  for IronPython, please send an email to ironpy@microsoft.com.
-#  By using this source code in any fashion, you are agreeing to be bound by
-#  the terms of the Shared Source License for IronPython.
+# This source code is subject to terms and conditions of the Microsoft Public
+# License. A  copy of the license can be found in the License.html file at the
+# root of this distribution. If  you cannot locate the  Microsoft Public
+# License, please send an email to  dlr@microsoft.com. By using this source
+# code in any fashion, you are agreeing to be bound by the terms of the 
+# Microsoft Public License.
 #
-#  You must not remove this notice, or any other, from this software.
+# You must not remove this notice, or any other, from this software.
 #
-######################################################################################
+#####################################################################################
 
 from lib.assert_util import *
 import IronPython.CodeDom
@@ -98,7 +98,10 @@ class assignStmt(object):
 class returnStmt(object):
     def __init__(self, val):
         self.value = val
-    def generate(self, indent): return '%sreturn %s\r\n' % ('    '*indent, self.value)
+    def generate(self, indent): 
+        if self.value != None:
+            return '%sreturn %s\r\n' % ('    '*indent, self.value)
+        return '%sreturn\r\n' % ('    '*indent)
 
 class eventAttachStmt(object):
     def __init__(self, lhs, rhs):
@@ -162,7 +165,7 @@ def RunGeneratedExe(file):
     oldDir = System.Environment.CurrentDirectory
     try:
         System.Environment.CurrentDirectory = targetdir
-        retval = nt.spawnl(0, file)
+        retval = nt.spawnl(0, file, file)
     except Exception, e:
         print "exception while running: %s" %e
         retval = 1
@@ -178,6 +181,8 @@ testCases = [
 #        [fieldMember('foo', 'bool')],
 #        ),        
 
+    fileGen([importStmt('clr', '*'), typeGen('foo', 'object', [methGen('test1', ('x','y','z'), [ returnStmt('a') ], 'int', ('Self()', 'int', 'int')), ] ) ]),
+
     fileGen( [importStmt('clr', '*'), 
             typeGen('foo', 'object', [    
                 methGen('test1', ('self',), [
