@@ -1,17 +1,18 @@
 #####################################################################################
 #
-#  Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation. 
 #
-#  This source code is subject to terms and conditions of the Shared Source License
-#  for IronPython. A copy of the license can be found in the License.html file
-#  at the root of this distribution. If you can not locate the Shared Source License
-#  for IronPython, please send an email to ironpy@microsoft.com.
-#  By using this source code in any fashion, you are agreeing to be bound by
-#  the terms of the Shared Source License for IronPython.
+# This source code is subject to terms and conditions of the Microsoft Public
+# License. A  copy of the license can be found in the License.html file at the
+# root of this distribution. If  you cannot locate the  Microsoft Public
+# License, please send an email to  dlr@microsoft.com. By using this source
+# code in any fashion, you are agreeing to be bound by the terms of the 
+# Microsoft Public License.
 #
-#  You must not remove this notice, or any other, from this software.
+# You must not remove this notice, or any other, from this software.
 #
-######################################################################################
+#####################################################################################
+
 
 from lib.assert_util import *
 items = 0
@@ -741,4 +742,94 @@ def test_eval_locals_simple():
     locs = Locals()
     AreEqual(eval("unknownvariable", globals(), locs), 'abc')
 
+
+def test_key_error():
+    class c: pass
+    class d(object): pass
+    
+    
+    for key in ['abc', 1, c(), d(), 1.0, 1L]:
+        try:
+            {}[key]
+        except KeyError, e:
+            AreEqual(e.args[0], key)
+        
+        try:
+            del {}[key]
+        except KeyError, e:
+            AreEqual(e.args[0], key)
+            
+        try:
+            set([]).remove(key)
+        except KeyError, e:
+            AreEqual(e.args[0], key)
+
+#CodePlex Work Item 7426
+@skip("cli")
+def test_contains():
+    class ContainsDict(dict):
+        was_called = False
+        def __contains__(self, key):
+            ContainsDict.was_called = True
+            return dict.__contains__(self, key)
+
+    md = ContainsDict()
+    md["stuff"] = 1
+    
+    AreEqual(ContainsDict.was_called, False)
+    AreEqual("nothing" in md, False)
+    AreEqual("stuff" in md, True)
+    AreEqual(ContainsDict.was_called, True)
+
+
+def test_stdtypes_dict():
+    temp_types = [  int,
+                    long,
+                    float,
+                    complex,
+                    bool,
+                    str,
+                    unicode,
+                    basestring,
+                    list,
+                    tuple,
+                    xrange,
+                    dict,
+                    set,
+                    frozenset,
+                    type,
+                    object,
+                    file ] #+ [eval("types." + x) for x in dir(types) if x.endswith("Type")]
+    
+    temp_keys = [ None, -1, 0, 1, 2.34, "", "None", int, object, test_stdtypes_dict, [], (None,)]
+    
+    for temp_type in temp_types:
+        for temp_key in temp_keys:
+            try:
+                temp_type.__dict__[temp_key] = 0
+                raise "Should have been an exception for " + str(temp_type)
+            except TypeError, e:
+                pass
+            except Exception, e:
+                print "Failed on", temp_type, "type using", temp_key, "as the key:", e
+                #CodePlex Work Item 8892
+                if temp_key==None: continue
+                raise e
+    
+#CodePlex Work Item 5712    
+def test_main_dict():
+    import __main__
+    #just make sure this doesn't throw...
+    t_list = []
+    for w in __main__.__dict__: t_list.append(w)
+    
+    #CodePlex Work Item 8961
+    if not is_cli:
+        t_list.sort()
+        g_list = globals().keys()
+        g_list.sort()
+        AreEqual(t_list, g_list)
+    
+    
+        
 run_test(__name__)
\ No newline at end of file
